package com.qmth.themis.backend.constant;

import com.qmth.themis.backend.config.DictionaryConfig;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.enums.UploadFileEnum;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 后台系统常量
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/15
 */
public class BackendConstant {

    /**
     * 获取系统上传环境
     *
     * @param type
     * @return
     */
    public static Map<String, Object> getAttachmentEnv(Integer type) {
        if (Objects.isNull(type) || Objects.equals(type, "")) {
            throw new BusinessException(ExceptionResultEnum.ATTACHMENT_TYPE_IS_NULL);
        }
        DictionaryConfig dictionaryConfig = SpringContextHolder.getBean(DictionaryConfig.class);
        Map<String, Object> mapParameter = new HashMap<>();
        mapParameter.put(SystemConstant.END_POINT, dictionaryConfig.aliYunOssDomain().getEndpoint());
        mapParameter.put(SystemConstant.ACCESS_KEY_ID, dictionaryConfig.aliYunOssDomain().getAccessKeyId());
        mapParameter.put(SystemConstant.ACCESS_KEY_SECRET, dictionaryConfig.aliYunOssDomain().getAccessKeySecret());
        mapParameter.put(SystemConstant.BUCKET, dictionaryConfig.aliYunOssDomain().getBucket());
        mapParameter.put(SystemConstant.OSS, dictionaryConfig.sysDomain().isOss());
        mapParameter.put(SystemConstant.ATTACHMENT_TYPE, dictionaryConfig.sysDomain().getAttachmentType());
        String uploadType = UploadFileEnum.convertToName(type);
        mapParameter.put(SystemConstant.UPLOAD_TYPE, uploadType);
        return mapParameter;
    }
    
    public static String getTempDir() {
        DictionaryConfig dictionaryConfig = SpringContextHolder.getBean(DictionaryConfig.class);
        return dictionaryConfig.sysDomain().getTempDataDir();
    }
}
